@@ -27,7 +27,7 @@ THE SOFTWARE.
 from pudb.settings import load_config
 
 
-NUM_VERSION = (2021, 1)
+NUM_VERSION = (2021, 2)
 VERSION = ".".join(str(nv) for nv in NUM_VERSION)
 __version__ = VERSION
 
@@ -173,7 +173,7 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False,
                     % status_msg),
                     urwid.Text("\n\nIf you decide to restart, this command "
                     "will be run prior to actually restarting:"),
-                    urwid.AttrMap(pre_run_edit, "value")
+                    urwid.AttrMap(pre_run_edit, "input", "focused input")
                     ])),
                 [
                     ("Restart", "restart"),
