@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 __copyright__ = """
 Copyright (C) 2009-2017 Andreas Kloeckner
 Copyright (C) 2014-2017 Aaron Meurer
@@ -25,27 +23,28 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
-
 import os
 import sys
 
-from pudb.py3compat import ConfigParser
-from pudb.lowlevel import lookup_module, get_breakpoint_invalid_reason
+from configparser import ConfigParser
+from pudb.lowlevel import (lookup_module, get_breakpoint_invalid_reason,
+                           settings_log)
 
 # minor LGPL violation: stolen from python-xdg
 
-_home = os.environ.get('HOME', None)
-xdg_data_home = os.environ.get('XDG_DATA_HOME',
-            os.path.join(_home, '.local', 'share') if _home else None)
+# see https://github.com/inducer/pudb/pull/453 for context
+_home = os.environ.get("HOME", os.path.expanduser("~"))
+xdg_data_home = os.environ.get("XDG_DATA_HOME",
+            os.path.join(_home, ".local", "share") if _home else None)
 
 
-XDG_CONFIG_HOME = os.environ.get('XDG_CONFIG_HOME',
-                                 os.path.join(_home, '.config') if _home else None)
+XDG_CONFIG_HOME = os.environ.get("XDG_CONFIG_HOME",
+                                 os.path.join(_home, ".config") if _home else None)
 
 if XDG_CONFIG_HOME:
     XDG_CONFIG_DIRS = [XDG_CONFIG_HOME]
 else:
-    XDG_CONFIG_DIRS = os.environ.get('XDG_CONFIG_DIRS', '/etc/xdg').split(':')
+    XDG_CONFIG_DIRS = os.environ.get("XDG_CONFIG_DIRS", "/etc/xdg").split(":")
 
 
 def get_save_config_path(*resource):
@@ -53,8 +52,11 @@ def get_save_config_path(*resource):
         return None
     if not resource:
         resource = [XDG_CONF_RESOURCE]
-    resource = os.path.join(*resource)
-    assert not resource.startswith('/')
+
+    # no idea what pylint's problem is here
+    resource = os.path.join(*resource)  # pylint: disable=no-value-for-parameter
+
+    assert not resource.startswith("/")
     path = os.path.join(XDG_CONFIG_HOME, resource)
     if not os.path.isdir(path):
         os.makedirs(path, 448)  # 0o700
@@ -71,7 +73,14 @@ SAVED_BREAKPOINTS_FILE_NAME = "saved-breakpoints-%d.%d" % sys.version_info[:2]
 BREAKPOINTS_FILE_NAME = "breakpoints-%d.%d" % sys.version_info[:2]
 
 
+_config_ = [None]
+
+
 def load_config():
+    # Only ever do this once
+    if _config_[0] is not None:
+        return _config_[0]
+
     from os.path import join, isdir
 
     cparser = ConfigParser()
@@ -85,11 +94,11 @@ def load_config():
         if cparser.has_section(CONF_SECTION):
             conf_dict.update(dict(cparser.items(CONF_SECTION)))
     except Exception:
-        pass
+        settings_log.exception("Failed to load config")
 
     conf_dict.setdefault("shell", "internal")
     conf_dict.setdefault("theme", "classic")
-    conf_dict.setdefault("line_numbers", False)
+    conf_dict.setdefault("line_numbers", "False")
     conf_dict.setdefault("seen_welcome", "a")
 
     conf_dict.setdefault("sidebar_width", 0.5)
@@ -99,20 +108,20 @@ def load_config():
 
     conf_dict.setdefault("current_stack_frame", "top")
 
-    conf_dict.setdefault("stringifier", "type")
+    conf_dict.setdefault("stringifier", "default")
 
     conf_dict.setdefault("custom_theme", "")
     conf_dict.setdefault("custom_stringifier", "")
     conf_dict.setdefault("custom_shell", "")
 
-    conf_dict.setdefault("wrap_variables", True)
+    conf_dict.setdefault("wrap_variables", "True")
     conf_dict.setdefault("default_variables_access_level", "public")
 
     conf_dict.setdefault("display", "auto")
 
-    conf_dict.setdefault("prompt_on_quit", True)
+    conf_dict.setdefault("prompt_on_quit", "True")
 
-    conf_dict.setdefault("jk_sidebar_scroll", False)
+    conf_dict.setdefault("hide_cmdline_win", "False")
 
     def normalize_bool_inplace(name):
         try:
@@ -121,13 +130,14 @@ def load_config():
             else:
                 conf_dict[name] = True
         except Exception:
-            pass
+            settings_log.exception("Failed to process config")
 
     normalize_bool_inplace("line_numbers")
     normalize_bool_inplace("wrap_variables")
     normalize_bool_inplace("prompt_on_quit")
-    normalize_bool_inplace("jk_sidebar_scroll")
+    normalize_bool_inplace("hide_cmdline_win")
 
+    _config_[0] = conf_dict
     return conf_dict
 
 
@@ -148,7 +158,7 @@ def save_config(conf_dict):
         cparser.write(outf)
         outf.close()
     except Exception:
-        pass
+        settings_log.exception("Failed to save config")
 
 
 def edit_config(ui, conf_dict):
@@ -167,8 +177,8 @@ def edit_config(ui, conf_dict):
     def _update_prompt_on_quit():
         pass
 
-    def _update_jk_sidebar_scroll():
-        pass
+    def _update_hide_cmdline_win():
+        ui.update_cmdline_win()
 
     def _update_current_stack_frame():
         ui.update_stack()
@@ -208,10 +218,10 @@ def edit_config(ui, conf_dict):
             conf_dict.update(new_conf_dict)
             _update_prompt_on_quit()
 
-        elif option == "jk_sidebar_scroll":
-            new_conf_dict["jk_sidebar_scroll"] = not check_box.get_state()
+        elif option == "hide_cmdline_win":
+            new_conf_dict["hide_cmdline_win"] = not check_box.get_state()
             conf_dict.update(new_conf_dict)
-            _update_jk_sidebar_scroll()
+            _update_hide_cmdline_win()
 
         elif option == "current_stack_frame":
             # only activate if the new state of the radio button is 'on'
@@ -255,19 +265,20 @@ def edit_config(ui, conf_dict):
             bool(conf_dict["prompt_on_quit"]), on_state_change=_update_config,
                 user_data=("prompt_on_quit", None))
 
-    cb_jk_sidebar_scroll = urwid.CheckBox("Use j/k keys for scrolling in "
-                                          "sidebar",
-            bool(conf_dict["jk_sidebar_scroll"]), on_state_change=_update_config,
-                user_data=("jk_sidebar_scroll", None))
+    hide_cmdline_win = urwid.CheckBox("Hide command line (Ctrl-X) window "
+                                      "when not in use",
+            bool(conf_dict["hide_cmdline_win"]), on_state_change=_update_config,
+                user_data=("hide_cmdline_win", None))
 
     # {{{ shells
 
     shell_info = urwid.Text("This is the shell that will be "
             "used when you hit '!'.\n")
-    shells = ["internal", "classic", "ipython", "bpython", "ptpython", "ptipython"]
+    shells = ["internal", "classic", "ipython", "ipython_kernel", "bpython",
+              "ptpython", "ptipython"]
     known_shell = conf_dict["shell"] in shells
     shell_edit = urwid.Edit(edit_text=conf_dict["custom_shell"])
-    shell_edit_list_item = urwid.AttrMap(shell_edit, "value")
+    shell_edit_list_item = urwid.AttrMap(shell_edit, "input", "focused input")
 
     shell_rb_group = []
     shell_rbs = [
@@ -297,7 +308,7 @@ def edit_config(ui, conf_dict):
 
     theme_rb_group = []
     theme_edit = urwid.Edit(edit_text=conf_dict["custom_theme"])
-    theme_edit_list_item = urwid.AttrMap(theme_edit, "value")
+    theme_edit_list_item = urwid.AttrMap(theme_edit, "input", "focused input")
     theme_rbs = [
             urwid.RadioButton(theme_rb_group, name,
                 conf_dict["theme"] == name, on_state_change=_update_config,
@@ -333,16 +344,19 @@ def edit_config(ui, conf_dict):
 
     # {{{ stringifier
 
-    stringifier_opts = ["type", "str", "repr"]
+    from pudb.var_view import STRINGIFIERS
+    stringifier_opts = list(STRINGIFIERS.keys())
     known_stringifier = conf_dict["stringifier"] in stringifier_opts
     stringifier_rb_group = []
     stringifier_edit = urwid.Edit(edit_text=conf_dict["custom_stringifier"])
-    stringifier_info = urwid.Text("This is the default function that will be "
-        "called on variables in the variables list.  Note that you can change "
-        "this on a per-variable basis by selecting a variable and hitting Enter "
-        "or by typing t/s/r.  Note that str and repr will be slower than type "
-        "and have the potential to crash PuDB.\n")
-    stringifier_edit_list_item = urwid.AttrMap(stringifier_edit, "value")
+    stringifier_info = urwid.Text(
+        "This is the default function that will be called on variables in the "
+        "variables list. You can also change this on a per-variable basis by "
+        "selecting a variable and typing 'e' to edit the variable's display "
+        "settings, or by typing one of d/t/r/s/i/c. Note that str and repr will "
+        "be slower than the default, type, or id stringifiers.\n")
+    stringifier_edit_list_item = urwid.AttrMap(stringifier_edit,
+                                               "input", "focused input")
     stringifier_rbs = [
             urwid.RadioButton(stringifier_rb_group, name,
                 conf_dict["stringifier"] == name,
@@ -423,14 +437,10 @@ def edit_config(ui, conf_dict):
 
     lb_contents = (
             [heading]
-            + [urwid.AttrMap(urwid.Text("Line Numbers:\n"), "group head")]
+            + [urwid.AttrMap(urwid.Text("General:\n"), "group head")]
             + [cb_line_numbers]
-
-            + [urwid.AttrMap(urwid.Text("\nPrompt on quit:\n"), "group head")]
             + [cb_prompt_on_quit]
-
-            + [urwid.AttrMap(urwid.Text("\nKeys:\n"), "group head")]
-            + [cb_jk_sidebar_scroll]
+            + [hide_cmdline_win]
 
             + [urwid.AttrMap(urwid.Text("\nShell:\n"), "group head")]
             + [shell_info]
@@ -517,14 +527,14 @@ def parse_breakpoints(lines):
         filename = None
         lineno = None
         cond = None
-        comma = arg.find(',')
+        comma = arg.find(",")
 
         if comma > 0:
             # parse stuff after comma: "condition"
             cond = arg[comma+1:].lstrip()
             arg = arg[:comma].rstrip()
 
-        colon = arg.rfind(':')
+        colon = arg.rfind(":")
         funcname = None
 
         if colon > 0:
@@ -575,11 +585,11 @@ def load_breakpoints():
     lines = []
     for fname in file_names:
         try:
-            rc_file = open(fname, "rt")
-        except IOError:
+            rc_file = open(fname)
+        except OSError:
             pass
         else:
-            lines.extend([l.strip() for l in rc_file.readlines()])
+            lines.extend([line.strip() for line in rc_file.readlines()])
             rc_file.close()
 
     return parse_breakpoints(lines)
@@ -593,8 +603,8 @@ def save_breakpoints(bp_list):
     if not save_path:
         return
 
-    histfile = open(get_breakpoints_file_name(), 'w')
-    bp_list = set([(bp.file, bp.line, bp.cond) for bp in bp_list])
+    histfile = open(get_breakpoints_file_name(), "w")
+    bp_list = {(bp.file, bp.line, bp.cond) for bp in bp_list}
     for bp in bp_list:
         line = "b %s:%d" % (bp[0], bp[1])
         if bp[2]:
