@@ -334,7 +334,7 @@ class VariableWidget(urwid.FlowWidget):
 
                     + text_width(var_label)
 
-                    + 2  # for ": "
+                    + 1  # for ":"
                     )
 
             _attr = [(apfx+"label", labellen), (apfx+"value", totallen - labellen)]
@@ -357,15 +357,15 @@ class VariableWidget(urwid.FlowWidget):
                             self.prefix+"  " + self.value_str]
 
                     attr = [
-                        [(apfx+"label", lprefix+text_width(self.var_label) + 1)],
-                        [(apfx+"value", lprefix+2+text_width(self.value_str))]
+                        [(apfx+"label", lprefix+text_width(self.var_label))],
+                        [(apfx+"value", lprefix+3+text_width(self.value_str))]
                         ]
                 else:
                     text = [self.prefix + self.var_label + ": " + self.value_str]
 
                     attr = [[
-                            (apfx+"label", lprefix+text_width(self.var_label)+2),
-                            (apfx+"value", text_width(self.value_str)),
+                            (apfx+"label", lprefix+text_width(self.var_label)+1),
+                            (apfx+"value", text_width(self.value_str)+1),
                             ]]
             else:
                 text = [self.prefix + self.value_str]
@@ -465,12 +465,16 @@ def get_stringifier(iinfo: InspectInfo) -> Callable:
     except KeyError:
         try:
             if not custom_stringifier_dict:  # Only execfile once
-                from os.path import expanduser
-                custom_stringifier_fname = expanduser(iinfo.display_type)
+                from os.path import expanduser, expandvars
+                custom_stringifier_fname = expanduser(expandvars(iinfo.display_type))
                 with open(custom_stringifier_fname) as inf:
                     exec(compile(inf.read(), custom_stringifier_fname, "exec"),
                             custom_stringifier_dict,
                             custom_stringifier_dict)
+        except FileNotFoundError:
+            ui_log.error("Unable to locate custom stringifier file {!r}"
+                         .format(iinfo.display_type))
+            return error_stringifier
         except Exception:
             ui_log.exception("Error when importing custom stringifier")
             return error_stringifier
